@@ -24,14 +24,20 @@
 #' @useDynLib GauPro, .registration = TRUE
 #' @importFrom Rcpp evalCpp
 #' @importFrom stats optim
-#' @keywords data, kriging, Gaussian process, regression
+# @keywords data, kriging, Gaussian process, regression
 #' @return Object of \code{\link{R6Class}} with methods for fitting GP model.
 #' @format \code{\link{R6Class}} object.
+#' @field D Number of input dimensions of data
 #' @examples
 #' #k <- GauPro_kernel$new()
 GauPro_kernel <- R6::R6Class(classname = "GauPro_kernel",
   public = list(
-
+    D = NULL
+    # k_diag = function(x) {
+    #   if (is.matrix(x)) {rep(self$s2, nrow(x))}
+    #   else if (length(x) == self$D) {self$s2}
+    #   else {stop("Error in k_diag #4928")}
+    # }
   ),
   private = list(
 
